import 'dart:math';

/// The main class that has to be instantiated.
///
/// Example:
/// ```
/// var chance = new Chance();
/// int number = chance.integer(min: 10, max: 20);
/// ```
class Chance {
  // Maximum integer that JS supports.
  static const _maxInt = 9007199254740992;
  static const _minInt = -_maxInt;
  final _random = new Random();

  /// Returns a random integer.
  ///
  /// Leave out [min] to get a value between 0 and [max].
  /// Leave out all arguments to get a value between [_minInt] and [_maxInt].
  int integer({int min, int max}) {
    // No arguments provided
    if (min == null && max == null) {
      min = _minInt;
      max = _maxInt;

    // Only max provided
    } else if (min == null && max != null)
      min = 0;

    // Only min provided
    else if (min != null && max == null)
      throw new StateError('Must not provide min without max');

    else if (min > max)
      throw new RangeError('Min must not be smaller than max.');

    return (_randomDouble * (max - min + 1) + min).floor();
  }

  /// Returns a random floating / double number.
  ///
  /// Leave out [min] to get a value between 0 and [max].
  /// Leave out all arguments to get a value between 0 and 1.
  double floating({double min, double max}) {
    // No arguments provided
    if (min == null && max == null) {
      min = 0.0;
      max = 1.0;

    // Only max provided
    } else if (min == null && max != null)
      min = 0.0;

    // Only min provided
    else if (min != null && max == null)
      throw new StateError('Must not provide min without max');

    else if (min > max)
      throw new RangeError('Min must not be smaller than max.');

    return _randomDouble * (max - min) + min;
  }

  /// Returns a random boolean value (true or false).
  ///
  /// The default likelihood of success (returning true) is 50%.
  /// Set an optional [likelihood] from 0 to 100.
  bool boolean({double likelihood = 50.0}) {
    if (likelihood > 100 || likelihood < 0)
      throw new RangeError('likelihood accepts values from 0 to 100');
    return _randomDouble * 100 < likelihood;
  }

  double get _randomDouble => _random.nextDouble();
}
